#include <stdio.h>
#include <sys/time.h>
#include <string.h>

#define BLOCK_SIZE 32

/* Blocked matrix multiply kernel (naive algorithm), using device global memory */
__global__ void MatMulKernel(const float *A, const float *B, float *C, const int n) {
	float Cvalue = 0;	// Accumulator for C value
	int row = /* TODO: compute index */;	// Row of the C matrix
	int col = /* TODO: compute index */;	// Column of the C matrix

	for (int e = 0; e < n; ++e)
		Cvalue += A[row*n+e] * B[e*n+col];	// Compute

	C[row*n+col] = Cvalue;	// Update C matrix
}

static int n;	// Length of matrix side (nb of elements)
static size_t size;	// Size of matrix (nb of bytes)

/* Kernel wrapper */
void MatMul(const float *A, const float *B, float *C) {
	/* Device memory areas */
	float *d_A, *d_B, *d_C;

	/* Could use better timing routines but the good old gettimeofday should serve our purpose  */
	struct timeval t1,t2;

	/* Begin timing */
	gettimeofday(&t1, NULL);

	/* Allocate device area for A and send host A to dev */
	/* TODO: allocate d_A */;
	/* TODO: send A to d_A */;

	/* Allocate device area for B and send host B to dev */
	/* TODO: allocate d_B */;
	/* TODO: send B to d_B */;

	/* Allocate device area for C */
	/* TODO: allocate d_C */;

	/* Setup Execution configuration */
	/* TODO: declare thread block settings */;
	/* TODO: declare block grid   settings */;

	/* Launch kernel */
	/* TODO: launch kernel */;

	/* Retrieve result */
	/* TODO: receive d_C into C */;

	/* Free device memory */
	/* TODO: free d_A, d_B, d_C */;

	/* End timing */
	gettimeofday(&t2, NULL);
	double t = (t2.tv_sec - t1.tv_sec) + 1e-6 * (t2.tv_usec - t1.tv_usec);
	printf("cu_global,%lu,%.2lf\n", (unsigned long)n, t);
}

int
main (int argc, char **argv) {
#if 0
    /* Uncomment and set device id to select another device */
    int device_id=0;
    cudaSetDevice(device_id);
#endif

	if (argc > 1)
		n = atoi(argv[1]);
	else
		n = 256;

	size = n*n*sizeof(float);

	float *A, *B, *C;
	A = (float *) malloc (size);
	B = (float *) malloc (size);
	C = (float *) malloc (size);

	for (int i = 0 ; i < n; i++) {
		for (int j = 0 ; j < n; j++) {
			A[i*n+j] = (float)i*j;
			B[i*n+j] = 1-(float)i*j;
		}
	}

	MatMul(A, B, C);

#if 0
	/* Enable for checking computation result */
	float *D = (float *) malloc (size);
	for (int i = 0 ; i < n; i++) {
		for (int j = 0 ; j < n; j++) {
			float d = 0;
			for (int k = 0 ; k < n; k++) {
				d += A[i*n + k] * B[k*n + j];
			}
			D[i*n+j] = d;
		}
	}
	for (int i = 0 ; i < n; i++) {
		for (int j = 0 ; j < n; j++) {
			float cv = C[i*n+j];
			float dv = D[i*n+j];
			float diff = (dv*dv - cv*cv)/(1+cv*dv);
			if (diff > 1e-4 || diff < -1e-4) {
				fprintf(stderr, "check failed\n");
				exit(1);
			}
		}
	}
	free(D);
#endif

	free(A);
	free(B);
	free(C);

	return 0;
}

